local mod = MorgDKP2:NewModule("LDBPlugin")
local L = LibStub("AceLocale-3.0"):GetLocale("MorgDKP2")
local C = LibStub("AceConfigDialog-3.0")
local broker = LibStub("LibDataBroker-1.1", true)
	
local mdkp = MorgDKP2

--locals


local defaults = { 
	profile = {
		
   	},
}

local options = {
		type = "group",
		handler = mod,
		name = L["LDB Plugin"],
		desc = L["LDB Plugin"],
		order = 1100,
		args = {
			toggle = {
				type = "toggle",
				name = L["Toggle LDB Plugin"],
				desc = L["Plugin frame will not disappear until UI reloaded."],
				get = function() return mdkp.db.profile.modules.LDBPlugin end,
				set = 	function(info, v) 
						mdkp.db.profile.modules.LDBPlugin = v
						if v then 
							mod:Enable()
							mod:debug("Enabled LDB")
						else
							mod:Disable()
							mod:debug("Disabled LDB")
						end
					end,
			},
        	},
}

mod.modName = L["LDB Plugin"]
mod.modref = "LDBPlugin"

function mod:GetOptions()
	return options
end

function mod:IsDKP()
	return nil
end

function mod:OnInitialize()
	self.db = mdkp.db:RegisterNamespace("LDBPlugin", defaults)
	
end

function mod:OnEnable()
	mdkp.db.profile.moduleON["LDBPlugin"] = true
	if not broker then 
		mdkp.db.profile.modules.LDBPlugin = nil
		mod:Disable()
		self:debug("No LDB")
		return
	end
	self:CreateLDBObject()
	
end

function mod:OnDisable()
	mdkp.db.profile.moduleON["LDBPlugin"] = nil
	--broker.UnregisterCallback(self, "LibDataBroker_AttributeChanged_LDBplugin")
	--self.LDBframe:Hide()
end

function mod:ActivateCore()
	if not self.LDBframe then return end
	mod.LDBframe.obj.icon = "Interface\\AddOns\\MorgDKP2\\Icons\\AquaON"
	mod.LDBframe.hint:SetText(L["|c000070ddClick:|r MorgBid2 Base query.\n|c000070ddALT-Click:|r Toggle ML/DE\n|c000070ddSHIFT-Click:|r Invite & Waitlist.\n|c000070ddCTRL-Click:|r Raid Tracker.\n|c000070ddCTRL-Click Itemlink:|r Ignore item."])
	self:ToggleFrameUpdate("TooltipLDB", true)
end

function mod:DeActivateCore()
	if not self.LDBframe then return end
	mod.LDBframe.obj.icon = "Interface\\AddOns\\MorgDKP2\\Icons\\AquaOFF"
	mod.LDBframe.hint:SetText(L["On STANDBY.\n|c000070ddClick:|r to enable."])
	self:ToggleFrameUpdate("TooltipLDB", true)
end

function mod:CreateLDBObject()
	if self.LDBframe then return end
	local onenter = function(f) f:Show() end
	local onleave = function(f) f:Hide() end
	self.LDBframe = self:Createframe("Tooltip_BrokerMorgDKP2", onenter, onleave, nil, nil, L["On STANDBY.\n|c000070ddClick:|r to enable."], "TooltipLDB",nil, nil, L["MorgDKP2"])
	self.LDBframe:SetFrameStrata("FULLSCREEN")
	self.LDBframe.obj = LibStub("LibDataBroker-1.1"):NewDataObject("Broker_MorgDKP2", {
		icon = "Interface\\AddOns\\MorgDKP2\\Icons\\AquaOFF",
		label = "MorgDKP2",
		OnClick = function(frame, button)
				if button == "RightButton" then
					if not C.OpenFrames["MorgDKP2"] then C:Open("MorgDKP2") 
					else C:Close("MorgDKP2") end
				else 
					if mod:CoreStatus() then
						if IsAltKeyDown() then
							mdkp.MLmode = not mdkp.MLmode
						elseif IsShiftKeyDown() then
							local core = self:GetModuleRef("CoreModule")
							core:ListDKP("All")
						elseif IsControlKeyDown() then
							local raidtracker = self:GetModuleRef("RaidTracker")
							if raidtracker then raidtracker:OpenFrame() end
						else 
							local core = self:GetModuleRef("CoreModule")
							core:MorgBidQuery()
						end
					else 
						self.LDBframe:Hide()
						mdkp:ActivateCore()
					end
					self:Updatetooltipdata(self.LDBframe)
				end
			end,
		OnEnter = function(frame)
				local X, Y = self:GetScaledCursorPosition()
				if Y < GetScreenHeight() / 2 then
					point, relPoint = "BOTTOMLEFT", "TOPLEFT"
				else
					point, relPoint = "TOPLEFT", "BOTTOMLEFT"
				end
				self.LDBframe:SetScript("OnUpdate", nil)
				self.LDBframe:ClearAllPoints()
    				self.LDBframe:SetPoint(point, frame, relPoint)
				self.LDBframe:Show()
				self:FrameUpdate(self.LDBframe)
			end,
		OnLeave = function()
				self.LDBframe:Hide() 
			end,
	})

end
