local mod = MorgDKP2:NewModule("Tracker", "AceEvent-3.0", "AceTimer-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("MorgDKP2")
local media = LibStub("LibSharedMedia-3.0", true)
local BB = LibStub("LibBabble-Boss-3.0"):GetLookupTable()

local mdkp = MorgDKP2
mod.modName = "Tracker"

--locals
local LL = LibStub("AceLocale-3.0"):NewLocale("Tracker", "enUS", true)
	
if LL then
	LL["DKP Tracking"] = true
	LL["Core of all MorgDKP2 raid tracking features. \nCan not be disabled."] = true
	LL["Start Event"] = true
	LL["Use start event when raid begins."] = true
	LL["Start Raid"] = true
	LL["Starts a new raid."] = true
	LL["End Raid"] = true
	LL["Finalizes the current raid."] = true
	LL["Export Raids"] = true
	LL["Exports current raid(s) data for import into website."] = true
	LL["Raid Tracker"] = true
	LL["Opens the raidtracker interface."] = true
	LL["Single run event"] = true
	LL["Use one event for entire raid.  Boss kills will not be tracked separately."] = true
	LL["Eqdkp+ event format"] = true
	LL["Remove the ' character from any event names because eqdkp+ is dumb."] = true
	LL["%s recorded at %s with %s attendees."] = true
	LL["%s  defeated in %s:%s!"] = true
	LL["End %s"] = true
	LL["and add these attendees < %s min?"] = true
	LL["No current raid exists."] = true
	LL["Removed %s raid attendees."] = true
	LL["Attendance reward"] = true
	LL["Minimum time to add member to raid and award run DKP."] = true
	LL["General options"] = true
	LL["Shortcuts"] = true
	LL["DKP listener"] = true
	LL["When you are in a tracked zone you will track DKP but you will not get the loot popup window or link to the raid.  You will act like a backup raidlog."] = true
	LL["Aliases"] = true
	LL["Main"] = true
	LL["Alt"] = true
	LL["ADD"] = true
	LL["Add alt to database."] = true
	LL["DKP Export"] = true
	LL["Clear the raid database?"] = true
	LL["YES"] = true
	LL["NO"] = true
	LL["CANCEL"] = true
	LL["Yes & Add"] = true
end

local LL = LibStub("AceLocale-3.0"):GetLocale("Tracker")

local fmt = string.format
local tinsert = table.insert
local tsort   = table.sort
local tremove = table.remove
local tostring = tostring 
local lower = string.lower
local gsub = string.gsub
local sub = string.sub
local floor = math.floor
local Player = UnitName("player")
local db

--statics
StaticPopupDialogs["BossAttempt2"] = {
	text = TEXT(L["Record boss attempt for %s?"]),
	button1 = TEXT(YES),
	button2 = TEXT(NO),
	OnAccept = function()
		mod:CustomEvent()
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1,
}

local IgnoreMobs = {
	[L["Son of Flame"]] = true,
	[L["Onyxian Whelp"]] = true,
	[L["Kil'rek"]] = true,
	[L["Amani Dragonhawk Spirit"]] = true,
}

local defaults = { 
	profile = {
		eqdkp = nil,
		runrewardtime = 1800,
   		
   	},
}

local options = {
		type = "group",
		handler = mod,
		name = LL["DKP Tracking"],
		desc = LL["Core of all MorgDKP2 raid tracking features. \nCan not be disabled."],
		order = 100,
		args = {
			head = {
				type = "header",
				name = LL["General options"],
				order = 10
			},
			usestart = {
				type = "toggle",
				name = LL["Start Event"],
				desc = LL["Use start event when raid begins."],
				get = function() return mdkp.db.profile.startrun end,
				set = function(info, v) mdkp.db.profile.startrun = v end,
				order = 100
			},	
			onerun = {
				type = "toggle",
				name = LL["Single run event"],
				desc = LL["Use one event for entire raid.  Boss kills will not be tracked separately."],
				get = function() return mdkp.db.profile.OneEvent end,
				set = function(info, v) mdkp.db.profile.OneEvent = v end,
				order = 105
			},
			eqdkpplus = {
				type = "toggle",
				name = LL["Eqdkp+ event format"],
				desc = LL["Remove the ' character from any event names because eqdkp+ is dumb."],
				get = function() return mod.db.profile.eqdkp end,
				set = function(info, v) mod.db.profile.eqdkp = v end,
				order = 110
			},
			dkplisten = {
				type = "toggle",
				name = LL["DKP listener"],
				desc = LL["When you are in a tracked zone you will track DKP but you will not get the loot popup window or link to the raid.  You will act like a backup raidlog."],
				get = function() return mod.db.profile.dkplistener end,
				set = function(info, v) mod.db.profile.dkplistener = v end,
				order = 115
			},
			rewardtime = {
				type = "range",
				name = LL["Attendance reward"],
				desc = LL["Minimum time to add member to raid and award run DKP."],
				get = function() return floor(mod.db.profile.runrewardtime / 60) end,
				set = function(info, v) mod.db.profile.runrewardtime = floor(v * 60) end,
				min = 0, 
				max = 120, 
				step = 1,
				order = 120
			},
			head1 = {
				type = "header",
				name = LL["Shortcuts"],
				order = 200
			},
			startrd = {
				type = "execute",
				name = LL["Start Raid"],
				desc = LL["Starts a new raid."],
				func = function() mod:BeginRaid() end,
				disabled = function() return not mod:IsEnabled() or mdkp.db.profile.InRaid end,
				order = 201
			},
			endrd = {
				type = "execute",
				name = LL["End Raid"],
				desc = LL["Finalizes the current raid."],
				func = function() mod:ShowEndRaidFrame() end,
				disabled = function() return not mod:IsEnabled() or not mdkp.db.profile.InRaid end,
				order = 202
			},
			runbonus = {
				type = "execute",
				name = LL["Export Raids"],
				desc = LL["Exports current raid(s) data for import into website."],
				func = function() mod:ExportRaids() end,
				disabled = function() return not mod:IsEnabled() end,
				order = 203
			},
			raidtracker = {
				type = "execute",
				name = LL["Raid Tracker"],
				desc = LL["Opens the raidtracker interface."],
				func = function() 
						local raidtracker = mdkp:GetModule("RaidTracker", true)
						if raidtracker then raidtracker:OpenFrame() end
				 end,
				disabled = function() return not mod:IsEnabled() or not mdkp.db.profile.moduleON.RaidTracker end,
				order = 204
			},
			aliases = {
				type = "group",
				handler = mod,
				name = LL["DKP Tracking"],
				desc = LL["Core of all MorgDKP2 raid tracking features. \nCan not be disabled."],
				order = 100,
				args = {
					head2 = {
						type = "header",
						name = LL["Aliases"],
						order = 300
					},
					alt = {
						type = "input",
						name = LL["Alt"],
						get = function() return (mod.alt or "") end,
						set = 	function(info, v) mod.alt = v end,
						order = 310
					},
					main = {
						type = "input",
						name = LL["Main"],
						get = function() return (mod.main or "") end,
						set = 	function(info, v) mod.main = v end,
						order = 320
					},
					addalt = {
						type = "execute",
						name = LL["ADD"],
						desc = LL["Add alt to database."],
						func = function() mod:AddAlt() end,
						order = 330
					},
				},
			},
        	},
}


function mod:GetOptions()
	return options
end

function mod:IsDKP()
	return nil
end

function mod:OnInitialize()
	self.db = mdkp.db:RegisterNamespace("Tracker", defaults)
	
end

function mod:OnEnable()
	db = mdkp.db.profile
	db.moduleON["Tracker"] = true
	if db.InRaid then self:StartAttEvent() end
	self.killed = {}
	self.updateframe = nil
	self.alt = nil
	self.main = nil
	
	--events
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "CombatLogEvent")
	self:RegisterEvent("CHAT_MSG_MONSTER_YELL", "HostileYell")
	self:RegisterEvent("CHAT_MSG_MONSTER_EMOTE", "HostileEmote")
	
	self:TogglePlayerDies(db.attemptmode)
	
end

function mod:OnDisable()
	self:CancelTimer(self.timer, true)
	db.moduleON["Tracker"] = nil 
end

--temp alias adding til raid browser done
function mod:AddAlt()
	if not self.alt or not self.main then return end
	local alt = string.utf8upper(sub(self.alt, 1, 1)) .. string.utf8lower(sub(self.alt, 2, -1))
	local main = string.utf8upper(sub(self.main, 1, 1)) .. string.utf8lower(sub(self.main, 2, -1))
	local core = self:GetModuleRef("CoreModule")
	if core then 
		core:AddAlias(alt, main)
		mod.alt = nil
		mod.main = nil 
	end
end

function mod:StartAttEvent()
	self:debug("attendance check ON")
	self.timer = self:ScheduleRepeatingTimer("CheckAttendance", 60)
end

function mod:CombatLogEvent(_, timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
	if not db.InRaid then return end
	--self:debug("Combat event.."..eventtype..","..(srcName or "nil")..","..(srcFlags or "nil")..","..(dstName or "nil")..","..(dstFlags or "nil"))
	if eventtype == "UNIT_DIED" or eventtype == "UNIT_DESTROYED" then self:HostileDeath(dstName) end
end

function mod:HostileDeath(mobName)
	--self:debug("Kill..."..mobName)
	local killTime = date("%m/%d/%y %H:%M:%S")
	local RAID = db.raidnum
	if not IgnoreMobs[mobName] and RAID > 0 then db.raidlog.lastmob = mobName end
 	if mobName ~= nil  then
  		local Boss
   		if mobName == L["Romulo"] or mobName == L["Julianne"] then 
   			Boss = L["Romulo & Julianne"]
 			self.killed[Boss] = self.killed[Boss] + 1 
  			if self.killed[Boss] < 4 or (RAID > 0 and db.raidlog[RAID].lastboss == Boss) then return end
  		elseif mobName == BB["Essence of Anger"] then Boss = BB["Reliquary of Souls"]
  		elseif mobName == BB["Lady Sacrolash"] or mobName == BB["Grand Warlock Alythess"] then
  			Boss = BB["Eredar Twins"]
  			self.killed[Boss] = self.killed[Boss] + 1
  			if self.killed[Boss] < 2 then return end
  		elseif mobName == BB["Gathios the Shatterer"] then
  			Boss = BB["Illidari Council"]
  		elseif mobName == BB["Kalecgos"] or mobName == BB["Sathrovarr the Corruptor"] then
  			Boss = BB["Kalecgos"]
  			if not self.killed[Boss] then self.killed[Boss] = 1
  			else self.killed[Boss] = self.killed[Boss] + 1 end
  			if self.killed[Boss] < 2 then return end
  		else
  			if not mdkp.ZoneBosses[db.menuzone] then return end
  			for _, bossname in pairs(mdkp.ZoneBosses[db.menuzone]) do
  				if mobName == bossname then Boss = mobName break end
  			end
  		end
  		if Boss then 
  			local bossformat
  			local eventnaming = self:GetModuleRef("Eventnaming")
			if eventnaming then _, _, bossformat = eventnaming:GetEventFormat() 
			else bossformat = "<boss>" end
			local eventname = self:GetEventName(Boss, bossformat)
  			self:LogBossKill(killTime, eventname, nil, Boss) 
  			local wait = self:GetModuleRef("Waitlist")
			if wait then wait:AwardWaitlistDKP(killTime, Boss) end
  		end
  	end
end

function mod:HostileYell()
	--self:debug("YELL")
	if not db.InRaid then return end
	if arg1 == L["Impossible! Stay your attack, mortals... I submit! I submit!"]  then	--majordomo
  		local killTime = date("%m/%d/%y %H:%M:%S")
  		local Boss = BB["Majordomo Executus"]
  		if db.raidnum > 0 then db.raidlog.lastmob = BB["Majordomo Executus"] end
 		local bossformat
 		local eventnaming = self:GetModuleRef("Eventnaming")
		if eventnaming then _, _, bossformat = eventnaming:GetEventFormat() 
		else bossformat = "<boss>" end
		local eventname = self:GetEventName(Boss, bossformat)
 		self:LogBossKill(killTime, Eventname, nil, Boss)
  	elseif arg1 == L["What devil art thou, that dost torment me thus?"] then	--R&J start
   		self.killed[L["Romulo & Julianne"]] = 0
   	elseif arg1 == L["These are the hallmarks..."] then
   		self.killed[BB["Eredar Twins"]] = 0
   	elseif arg1 == L["No longer will I be a slave to Malygos! Challenge me and you will be destroyed!"] then	--Kalecgos start
   		self.killed[BB["Kalecgos"]] = 0
  	elseif arg1 == L["The time is now! Leave none standing! "] then self:ChangeLootMethod("freeforall") 	--vashj P2
  	elseif arg1 == L["You may want to take cover. "] then self:ChangeLootMethod("master") 			--vashj P3
  	elseif arg1 == L["The Legion's final conquest has begun! Once again the subjugation of this world is within our grasp. Let none survive!"] then self:ChangeLootMethod("master") 	--rage
  	elseif arg1 == L["You are defenders of a doomed world! Flee here, and perhaps you will prolong your pathetic lives!"] then self:ChangeLootMethod("master") 				--anatheron
  	elseif arg1 == L["Cry for mercy! Your meaningless lives will soon be forfeit!"] then self:ChangeLootMethod("master") 									--kazrogal
  	elseif arg1 == L["Abandon all hope! The Legion has returned to finish what was begun so many years ago. This time, there will be no escape!"] then self:ChangeLootMethod("master") 	--azgalore
  	end
end

function mod:HostileEmote()
	if not db.InRaid then return end
	if  arg1 == L["The halls of Karazhan shake, as the curse binding the doors of the Gamesman's Hall is lifted."]  then	--chess
  		local killTime = date("%m/%d/%y %H:%M:%S")
 		local Boss = BB["Chess Event"]
 		if db.raidnum > 0 then db.raidlog.lastmob = BB["Chess Event"] end
 		local bossformat
 		local eventnaming = self:GetModuleRef("Eventnaming")
		if eventnaming then _, _, bossformat = eventnaming:GetEventFormat() 
		else bossformat = "<boss>" end
		local eventname = self:GetEventName(Boss, bossformat)
		self:LogBossKill(killTime, eventname, nil, Boss)
	end
end

function mod:TogglePlayerDies(switch)
	if switch then self:RegisterEvent("PLAYER_DEAD", "LeaderDied")
	else self:UnregisterEvent("PLAYER_DEAD") end
end

function mod:LeaderDied()
	if not db.InRaid then return end
	StaticPopup_Show ("BossAttempt2", db.custom)
end

function mod:BeginRaid(frame)
	self:debug("Begin raid")
	if not self:IsRaided() then return end
	--self:RequestLeader()
	local StartDate = date("%m/%d/%y %H:%M:%S")
	local ZoneName = GetRealZoneText()
	local Startname, Runname, Startnote, Runnote, startformat, trashformat
	
	--initialise new raid database
	if not db.raidnum then db.raidnum = 0 end
	db.raidnum = db.raidnum + 1
	local raidnum = db.raidnum
	db.raidlog[raidnum] = { }
	db.raidlog[raidnum].playerinfo = { }
	db.raidlog[raidnum].join = { }
	db.raidlog[raidnum].leave = { }
	db.raidlog[raidnum].bosskills = { }
	db.raidlog[raidnum].loot = { }
	db.raidlog[raidnum].zone = ZoneName
	db.raidlog[raidnum].raidstart = StartDate
	db.lootbuffer = { }
	db.raidlog.attendancedone = {}
	if not db.raidlog.dkpevents then db.raidlog.dkpevents = { } end
	if not db.raidlog.aliases then db.raidlog.aliases = { } end
	db.raidlog[raidnum].eqdkp = db.eqDKP[db.raid].eqDKPsite
	db.raidlog[raidnum].prefix = db.eqDKP[db.raid].prefix
	db.InRaid = true
	
	--clear player received loot flag
	for name, _ in pairs(db.info) do
		db.info[name].raidloot = 0
	end
	
	--update gear etc
	local core = self:GetModuleRef("CoreModule")
	if core then core:Scan() end 
		
	--module setup
	local dkpovertime = self:GetModuleRef("DKPovertime")
	if dkpovertime then dkpovertime:AtRaidStart() end
	local eventnaming = self:GetModuleRef("Eventnaming")
	if eventnaming then startformat, trashformat, _ = eventnaming:GetEventFormat() 
	else 
		startformat = "<zone> Start"
		trashformat = "<zone> Run"
	end
	
	--menu data
	db.raidstart = StartDate
	db.raidend = L["PENDING"]
	db.raidexport = L["PENDING"]
	
	--start event
	if db.startrun then
		Startnote = gsub(startformat, "<zone>", ZoneName, 1)
		Startname = self:GetEventName(Startnote, startformat)
		self:LogBossKill(StartDate, Startname, 1, Startnote, nil, 1) 
	end
	
	local wait = self:GetModuleRef("Waitlist")
	if wait then wait:AtRaidStart(StartDate, ZoneName) end
	
	--run event
	Runnote = gsub(trashformat, "<zone>", ZoneName, 1)
	Runname = self:GetEventName(Runnote, trashformat)
	self:LogBossKill(StartDate, Runname, 1, Runnote, 1, 2) 
	
	--schedule attendance check
	self.timer = self:ScheduleRepeatingTimer("CheckAttendance", 60)
	
	--add attendess to raidlog if not already added
	local attendees = { }
	local counter = 0
	for name in pairs(mdkp.roster) do
    		local race = UnitRace(name)
    		local level = UnitLevel(name)
		local guild = GetGuildInfo(name)
		local classtrue, englishclass = UnitClass(name)
		if not db.raidlog[raidnum].playerinfo[name] then 
			db.raidlog[raidnum].playerinfo[name] = { } 
			db.raidlog[raidnum].playerinfo[name].class = classtrue
			db.raidlog[raidnum].playerinfo[name].race = race 
			db.raidlog[raidnum].playerinfo[name].level = level 
			db.raidlog[raidnum].playerinfo[name].guild = guild 
		end
		db.raidlog[raidnum].join[name] = StartDate
		counter = counter + 1
		self:PlayerinDB(name, englishclass)
	end
	if frame then self:Updatetooltipdata(frame) end
end

function mod:EndRaid(frame)
	self:debug("End Raid")
	local db = mdkp.db.profile
	if not db.InRaid then self:out(LL["No current raid exists."]) return end
	local raidnum = db.raidnum
	local EndTime = date("%m/%d/%y %H:%M:%S")
	
	--end raid flags
	db.raidend = EndTime
	db.InRaid = nil
	
	--clear raid values
	db.raidlog.attendancedone = {}
	db.custom = L["NEW"]
	
	--module processing
	local wait = self:GetModuleRef("Waitlist")
	if wait then wait:AtRaidEnd() end
	local dkpovertime = self:GetModuleRef("DKPovertime")
	if dkpovertime then dkpovertime:AtRaidEnd() end
	
	--cancel attendance check
	self:CancelTimer(self.timer, true) 
	
	counter = 0
	if mdkp.roster then
		for name in pairs(mdkp.roster) do
    			db.raidlog[raidnum].leave[name] = EndTime
			counter = counter + 1
		end
		self:out(fmt(LL["Removed %s raid attendees."], counter))
	end
	if frame then self:Updatetooltipdata(frame) end
	self:ToggleFrameUpdate("TooltipFubar", true)
end

function mod:ShowEndRaidFrame(frame)
	self:debug("End raid frame")
	local db = mdkp.db.profile
	local id = self:GetEventID(2)
	local runname = db.raidlog[db.raidnum].bosskills[id].note
	attendees = {}
	for name, jointime in pairs(db.raidlog[db.raidnum].join) do
		if not db.raidlog.attendancedone or not db.raidlog.attendancedone[name] then
			table.insert(attendees, name)
		end
	end
	if not self.frame then self.frame = self:CreateAttendanceFrame(frame) end
	local f = self.frame
	f.endraid:SetText(fmt(LL["End %s"], runname))
	local width, height = 100, 50
	
   	local num = #attendees
	if num > 0 then
		local width2 = f.endraid:GetStringWidth() + 10
   		f.endraid2:SetText(fmt(LL["and add these attendees < %s min?"], self.db.profile.runrewardtime/60))
   		f.endraid2:Show()
   		attendees = self:ArraytoString(attendees)
		f.attends:SetText(attendees)
		f.attends:SetWidth(width2)
		f.attends:Show()
   		height = 100 + num * db.media.fontsize
		width = f.endraid2:GetStringWidth() + 30
		f.yesadd:Show()
   	else
		f.endraid:SetText(f.endraid:GetText() .. "?")
		width = f.endraid:GetStringWidth() + 30
		f.attends:Hide()
   		f.endraid2:Hide()
		f.yesadd:Hide()
   	end
	f:SetHeight(height)
	f:SetWidth(width)
	f:Show()
end

function mod:FillDKPChanges()
	local temp = { }
	for pool, data in pairs(db.raidlog.dkpevents) do
		for index, changes in pairs(data) do
			if changes.addtype == "Decay" then
				for key, name in pairs(changes.members) do
					name = self:CheckforAlias(name, pool)
					self:PointsPoolExists(name, pool)
					local value = 0 - (floor((db.info[name][pool].points * changes.value/100) + 0.5))
					tinsert(temp, { name = name, 
							value = value, 
							members = {[1] = name},
							eqdkp = changes.eqdkp,
							prefix = changes.prefix,
							addtype = "individual",
							event = changes.event})
				end
			else tinsert(temp, changes)
			end
		end
	end
	return temp
end

function mod:ExportRaids(frame) 
	self:OutputRaid(db.raidlog, nil, frame)
	db.raidexport = L["COMPLETED"]
	if frame then 
		self.updateframe = frame
		self:Updatetooltipdata(frame)
	else
		self:ToggleFrameUpdate("TooltipLDB")
	end
end

function mod:ClearRaidDB()
	db.raidlog = {}
	db.raidnum = 0
	db.lootbuffer = { }
	db.raidstart = L["PENDING"]
	db.raidend = L["PENDING"]
	db.raidexport = L["PENDING"]
	db.raidlog.dkpevents = {}
	db.raidlog.aliases = { }
	db.InRaid = nil
	self:out(L["Cleared the MorgDKP2 raid database."])
	self:Updatetooltipdata(self.updateframe)
	self.updateframe = nil
end

function mod:OutputRaid(raidlog, raidid)
	self:debug("Export Raid")
	local DKPChanges = self:FillDKPChanges()
	local link = "<MorgDKP2><version>".. mdkp.webversion .. "</version>"
	if raidlog.aliases then
		link = link.."<Aliases>"
		local index = 1
		for pool, data in pairs(raidlog.aliases) do
			tsort(data, function (a, b) return a.alt < b.alt end)
			for _, data2 in pairs(data) do
				link = link .."<key".. index ..">"	
				link = link .."<alt>".. data2.alt .."</alt>"
				link = link .."<main>".. data2.main .."</main>"
				link = link .."<eqdkp>".. data2.eqdkp .."</eqdkp>"
				link = link .."<prefix>".. db.eqDKP[pool].prefix .."</prefix>"
				link = link .."<action>".. data2.action .."</action>"
				link = link .."</key".. index ..">"
				index = index + 1
			end
		end
		link = link.."</Aliases>"
	end
	if DKPChanges then
		link = link.."<DKPChanges>"
		for key, data in pairs(DKPChanges) do
			link = link .."<key".. key ..">"	
			link = link .."<name>".. data.name .."</name>"
			link = link .."<value>".. data.value .."</value>"
			link = link .."<eqdkp>".. data.eqdkp .."</eqdkp>"
			link = link .."<prefix>".. data.prefix .."</prefix>"
			link = link .."<event>".. data.event .."</event>"
			link = link .."<addtype>".. data.addtype .."</addtype>"
			link = link .."<members>"
			for key2, val2 in pairs(data.members) do
				link = link .."<key".. key2 ..">" .. val2 .. "</key".. key2 ..">"
			end
			link = link .."</members>"
			link = link .."</key".. key ..">"
		end
		link = link .."</DKPChanges>"
	end
	link = link .."<Raids>"
	if db.raidnum > 0 then
		for raidindex = 1, db.raidnum do
			if not raidid or raidindex == raidid then
				local raiddata = raidlog[raidindex]
				link = link .."<key".. raidindex ..">"
				link = link .."<zone>".. raiddata.zone .."</zone>"
				link = link .."<date>".. raiddata.raidstart .."</date>"
				link = link .."<eqdkp>".. raiddata.eqdkp .."</eqdkp>"
				link = link .."<prefix>".. raiddata.prefix .."</prefix>"
				link = link .."<Attendees>"
				local index = 1
				for key, _ in pairs(raiddata.playerinfo) do
					link = link .."<key".. index ..">"
					link = link .."<name>".. key .."</name>"
					for key2, val2 in pairs(raiddata.playerinfo[key]) do
						link = link .."<".. key2 ..">".. val2 .."</".. key2 ..">"
					end
					link = link .."</key".. index ..">"
					index = index + 1
				end
				link = link .."</Attendees>"
				link = link .."<Bosskills>"
				for key, data in pairs(raiddata.bosskills) do
					if not db.OneEvent or (db.OneEvent and data.trash ~= 0) then
						link = link .."<key".. key ..">"
						link = link .."<name>".. data.name .."</name>"
						link = link .."<note>".. data.note .."</note>"
						link = link .."<time>".. data.time .."</time>"
						link = link .."<value>".. data.value .."</value>"
						link = link .."<trash>".. data.trash .."</trash>"
						link = link .."<attendees>"
						for key2, val2 in pairs(data.attendees) do
							link = link .."<key".. key2 ..">" .. val2 .. "</key".. key2 ..">"
						end
						link = link .."</attendees>"
						link = link .."</key".. key ..">"
					end
				end
				link = link .."</Bosskills>"
				link = link .."<Loot>"
				for key, data in pairs(raiddata.loot) do
					link = link .."<key".. key ..">"
					for key2, val2 in pairs(data) do
						link = link .."<".. key2 ..">".. val2 .."</".. key2 ..">"
					end
					link = link .."</key".. key ..">"
				end
				link = link .."</Loot>"
				link = link .."</key".. raidindex ..">"
				raidindex = raidindex + 1
			end
		end
	end
	link = link .."</Raids>"
	link = link .."</MorgDKP2>"	
	self:DKPOutFrame(link)
end

function mod:DKPOutFrame(link)
	self:debug("DKPOut frame..")
	if not self.exportframe then
		local background = media:Fetch("background", db.media.background)
		local border = media:Fetch("border", db.media.border)
		local font = media:Fetch("font", db.media.font)
		
		self.exportframe = CreateFrame("frame", nil, UIParent)
		local f = self.exportframe
		
		f:SetFrameStrata("DIALOG")
   		f:ClearAllPoints()
   		f:SetBackdrop({ bgFile = background, tile = false, tileSize = 16, edgeFile = border, edgeSize = 8, insets = { left = 6, right = 6, top = 6, bottom = 6 } })
		f:SetBackdropColor(unpack(db.media.backdropC))
		f:SetBackdropBorderColor(unpack(db.media.borderC))
		f:SetPoint("CENTER", UIParent, "CENTER")
		f:SetHeight(120)
		f:SetWidth(200)
		
		f.title = f:CreateFontString()
   		f.title:SetPoint("TOP", f, "TOP", 0, -10)
   		f.title:SetFont(font, 14)
   		f.title:SetText(LL["DKP Export"])
   		f.title:SetTextColor(unpack(db.media.borderC))
   		f.title:Show()
   	
   		f.edit = CreateFrame("EditBox", nil, f)
   		f.edit:ClearAllPoints()
   		f.edit:SetPoint("TOPLEFT", f, "TOPLEFT", 45, -30)
		f.edit:SetAutoFocus(true)
		f.edit:SetFont(font, 12)
		f.edit:SetHeight(30)
		f.edit:SetWidth(120)
		f.edit:SetBackdrop({ bgFile = background, tile = false, tileSize = 16, edgeFile = border, edgeSize = 16, insets = { left = 6, right = 6, top = 6, bottom = 6 } })
		f.edit:SetBackdropColor(unpack(db.media.backdropC))
		f.edit:SetBackdropBorderColor(unpack(db.media.borderC))
		f.edit:SetTextInsets(5, 5, 5, 5)
		f.edit:Show()
		
		f.ques = f:CreateFontString()
   		f.ques:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", -15, 35)
   		f.ques:SetFont(font, 14)
   		f.ques:SetText(LL["Clear the raid database?"])
   		f.ques:Show()
   		
   		f.NO = CreateFrame("Button", nil, f, "OptionsButtonTemplate")
		f.NO:SetWidth(40)
		f.NO:SetHeight(20)
		f.NO:ClearAllPoints()
		f.NO:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", -10, 10)
		f.NO:SetScript("OnClick", function(f) f:GetParent():Hide() end)
		f.NO:Show()
	
		f.NO.text = f.NO:CreateFontString()
   		f.NO.text:SetFont(font, 12)
   		f.NO.text:ClearAllPoints()
		f.NO.text:SetPoint("CENTER", f.NO, "CENTER")
		f.NO.text:SetText(LL["NO"])
		
		f.YES = CreateFrame("Button", nil, f, "OptionsButtonTemplate")
		f.YES:SetWidth(40)
		f.YES:SetHeight(20)
		f.YES:ClearAllPoints()
		f.YES:SetPoint("RIGHT", f.NO, "LEFT", -5)
		f.YES:SetScript("OnClick", function(f) f:GetParent():Hide() self:ClearRaidDB() end)
		f.YES:Show()
	
		f.YES.text = f.YES:CreateFontString()
   		f.YES.text:SetFont(font, 12)
   		f.YES.text:ClearAllPoints()
		f.YES.text:SetPoint("CENTER", f.YES, "CENTER")
		f.YES.text:SetText(LL["YES"])
	end
	local f = self.exportframe
	f.edit:SetText(link)
	f.edit:HighlightText()
	f:Show()
end

function mod:CustomEvent()
	self:debug("Custom event")
	if not db.InRaid then 
		self:out(LL["No current raid exists."])
		return 
	end
	local event = db.custom
	if event == L["NEW"] then event = event .. " Event"
	else event = event .. " Attempt" end
	self:LogBossKill(date("%m/%d/%y %H:%M:%S"), event, 1, event, nil, 3) 
end

function mod:LogBossKill(killTime, Boss, notboss, Note, quiet, id, attends, dkpvalue)
	self:debug("Boss kill "..Boss.." "..Note)
	local raidnum = db.raidnum
	local POOL = db.raid
	local counter
	--if not db.OneEvent or id == 2 then
		local attendees = { }
		if not attends then
			if id ~= 2 then
				counter = 0
				for name in pairs(mdkp.roster) do
    					table.insert(attendees, name)
					counter = counter + 1
				end
			end
		else
			counter = #attends
			attendees = attends
		end
		if self.db.profile.eqdkp then
			Boss = self:Removehash(Boss)
			Note = self:Removehash(Note)
		end
		local value = 0
		if db.eqDKP[POOL]["Events"][Boss] then value = db.eqDKP[POOL]['Events'][Boss].value 
		elseif attends then value = dkpvalue end
		table.insert(db.raidlog[raidnum].bosskills,
						{
							["name"] = Boss,
							["note"] = Note,
							["time"] = killTime,
							["attendees"] = attendees,					
							["value"] = value,
							["trash"] = id or 0
						}
					)
		if not quiet then
			self:out(fmt(LL["%s recorded at %s with %s attendees."], Note, killTime, counter))
			if id ~= 1 and id ~= 3 and mdkp.Starttime then
				local totaltime = self:TimeDiff(mdkp.Starttime, killTime)
				local min = floor(totaltime/60)
				local sec = totaltime - (min * 60)
				if sec < 10 then sec = "0"..sec end
				self:out(fmt(LL["%s  defeated in %s:%s!"], Note, min, sec))
			end
		end
		self:AwardDKP(nil, value, POOL, attendees, nil, quiet)
	--end
	if not notboss then 
		db.raidlog.lastboss = Boss
	end
	if Boss == BB["Rage Winterchill"] or Boss == BB["Anetheron"] or Boss == BB["Kaz'rogal"] then self:ChangeLootMethod("group") end
end

function mod:Removehash(event)
	event = gsub(event, "'", "")
	return event
end

function mod:ChangeLootMethod(method)
	if not self.lootmaster then self.lootmaster = Player end
	if method == "master" then SetLootMethod("master", self.lootmaster) 
  	else SetLootMethod(method) end
end

function mod:ItemRecorded(item, name)
	for _, ID in pairs(db.info[name].items) do
		if item == ID then
			return true
		end
	end
	return false
end

function mod:CreateAttendanceFrame(updateframe)
	local db = mdkp.db.profile
	local font = media:Fetch("font", db.media.font)
	local ondragstart = function(f) f:StartMoving() end
	local ondragstop =  function(f)
   					f:StopMovingOrSizing()
   					local point, _, relativePoint, x, y = f:GetPoint()
					mod.db.profile.attframe = {point = point, relpoint = relativePoint, x = x, y = y}
   				end
	local f = self:Createframe(nil, nil, nil, nil, nil, nil, "Attframe", nil, nil, nil, ondragstart, ondragstop)
	f:EnableKeyboard(1)
	f:ClearAllPoints()
	local point, relpoint, x, y
   	local db2 = self.db.profile.attframe
   	if db2 then point, relpoint, x, y = db2.point, db2.relpoint, db2.x, db2.y
   	else point, relpoint, x, y = "CENTER", "CENTER", 0, 0 end
   	f:SetPoint(point, UIParent, relpoint, x, y)
   		
	f.endraid = f:CreateFontString()
   	f.endraid:SetPoint("TOPLEFT", f, "TOPLEFT", 10, -10)
   	f.endraid:SetJustifyH("LEFT")
   	f.endraid:SetFont(font, db.media.fontsize)
   	f.endraid:SetTextColor(unpack(db.media.borderC))
   	f.endraid:Show()
   	
   	f.endraid2 = f:CreateFontString()
   	f.endraid2:SetPoint("TOPLEFT", f.endraid, "BOTTOMLEFT", 10, -5)
   	f.endraid2:SetJustifyH("LEFT")
   	f.endraid2:SetFont(font, db.media.fontsize)
   	f.endraid2:SetTextColor(unpack(db.media.borderC))
   	
   	f.attends = CreateFrame("EditBox", nil, f)
	f.attends:ClearAllPoints()
	f.attends:SetMultiLine(1)
	f.attends:SetAutoFocus(nil)
	f.attends:SetPoint("TOPLEFT", f.endraid2, "BOTTOMLEFT", 20, -10)
	f.attends:SetScript("OnEscapePressed", function(f) f:ClearFocus() end)
	f.attends:SetFont(font, db.media.fontsize - 1)
	f.attends:SetTextColor(1, 1, 1)
	f.attends:SetTextInsets(3, 3, 3, 3)
	f.attends:SetBackdrop({ bgFile = media:Fetch("background", db.media.background), tile = false, tileSize = 16, edgeFile = media:Fetch("border", db.media.border), edgeSize = 8, insets = { left = 3, right = 3, top = 3, bottom = 3 } })
	f.attends:SetBackdropColor(unpack(db.media.backdropC))
	f.attends:SetBackdropBorderColor(unpack(db.media.borderC))
   	
   	f.no = CreateFrame("Button", nil, f, "OptionsButtonTemplate")
   	f.no:SetScript("OnClick", function(f) f:GetParent():Hide() end)
   	f.no:SetWidth(40 * db.media.fontsize/8)
   	f.no:SetHeight(db.media.fontsize + 5)
   	f.no:ClearAllPoints()
   	f.no:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", -10, 10)
   	f.no:Show()
   	
   	f.no.text = f.no:CreateFontString()
   	f.no.text:SetFont(font, db.media.fontsize)
   	f.no.text:ClearAllPoints()
	f.no.text:SetPoint("CENTER", f.no, "CENTER")
	f.no.text:SetText(LL["CANCEL"])
	
   	f.yes = CreateFrame("Button", nil, f, "OptionsButtonTemplate")
   	f.yes:SetScript("OnClick", function(f) f:GetParent():Hide() mod:EndRaid(updateframe) end)
   	f.yes:SetWidth(30 * db.media.fontsize/8)
   	f.yes:SetHeight(db.media.fontsize + 5)
   	f.yes:ClearAllPoints()
   	f.yes:SetPoint("RIGHT", f.no, "LEFT", -5, 0)
   	f.yes:Show()
   	
   	f.yes.text = f.yes:CreateFontString()
   	f.yes.text:SetFont(font, db.media.fontsize)
   	f.yes.text:ClearAllPoints()
	f.yes.text:SetPoint("CENTER", f.yes, "CENTER")
	f.yes.text:SetText(LL["YES"])
	
   	f.yesadd = CreateFrame("Button", nil, f, "OptionsButtonTemplate")
   	f.yesadd:SetScript("OnClick", function(f) f:GetParent():Hide() mod:AddandEnd(updateframe) end)
   	f.yesadd:SetWidth(55 * db.media.fontsize/8)
   	f.yesadd:SetHeight(db.media.fontsize + 5)
   	f.yesadd:ClearAllPoints()
   	f.yesadd:SetPoint("RIGHT", f.yes, "LEFT", -5, 0)
   	
   	f.yesadd.text = f.yesadd:CreateFontString()
   	f.yesadd.text:SetFont(font, db.media.fontsize)
   	f.yesadd.text:ClearAllPoints()
	f.yesadd.text:SetPoint("CENTER", f.yesadd, "CENTER")
	f.yesadd.text:SetText(LL["Yes & Add"])
	
   	return f
end

function mod:AddandEnd(frame)
	local db = mdkp.db.profile
	local attendees =  self:AttendtoArray(self.frame.attends)
	if attendees then
		local id = self:GetEventID(2)
		local value = 0
		local POOL = db.raid
		if db.eqDKP[POOL]['Events'][db.raidlog[db.raidnum].bosskills[id].name] then
			value = db.eqDKP[POOL]['Events'][db.raidlog[db.raidnum].bosskills[id].name].value
		end
		for num, name in pairs(attendees) do
			if not db.raidlog.attendancedone or not db.raidlog.attendancedone[name]  then
				table.insert(db.raidlog[db.raidnum].bosskills[id].attendees, name)
			end
		end
		if value then self:AwardDKP(nil, value, POOL, attendees, nil, 1) end
	end
	self:EndRaid(frame)
end

function mod:CheckAttendance()
	--self:debug("Checking attendees...")
	if not db.InRaid then return end 
	local POOL = db.raid
	local raidnum = db.raidnum
	local runrewardtime = self.db.profile.runrewardtime
	local currtime = date("%m/%d/%y %H:%M:%S")
	
	--get event value
	local value = 0
	local id = self:GetEventID(2)
	if db.eqDKP[POOL]['Events'][db.raidlog[raidnum].bosskills[id].name] then
		value = db.eqDKP[POOL]['Events'][db.raidlog[raidnum].bosskills[id].name].value
	end
	
	--add player to run event if been here long enough
	for name, jointime in pairs(db.raidlog[raidnum].join) do
		if not db.raidlog.attendancedone[name] and not db.raidlog[raidnum].leave[name] then
			if self:TimeDiff(jointime, currtime)  >= runrewardtime then	
				table.insert(db.raidlog[raidnum].bosskills[id].attendees, name)
				db.raidlog.attendancedone[name]  = true
				self:debug("Added "..name.." to run.")
				if value then self:AwardDKP(nil, value, POOL, {[1] = name}, nil, 1) end
			end
		end
	end
	
	--waitlist check
	local waitdone
	local wait = self:GetModuleRef("Waitlist")
	if wait then waitdone = wait:AtAttendanceCheck() end
	
	--turn off attendance event if all done
	local attdone = true
	for name, jointime in pairs(db.raidlog[raidnum].join) do
		if not db.raidlog.attendancedone[name] then attdone = nil end
	end
	if attdone and (not wait or (wait and waitdone)) then self:CancelTimer(self.timer, true) end
end
